"""
Created for training neural-network model for correlation entropy

@author: Faluke Aikebaier
"""

### 1. Libraries
import matplotlib.pyplot as plt
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
from sklearn.model_selection import train_test_split

### 2. Data
#### 2.1 Data is obtained for n site Hubbard model by using get_data.py
loaded_data_X = np.load("my_data_Y.npy")
# Correlators: first 32 columns are two-point correlators, rest are four-point correlators
loaded_data_Y = np.load("my_data_Y.npy") # Correlation entropy
#### 2.2 train-test split
trainX, testX, trainY, testY=train_test_split(loaded_data_X, loaded_data_Y, test_size=0.2, random_state=42)
### use loaded_data_X[:,:32] to train the model with two point corrlators, change the input shape below as input_shape=[32,]
### use loaded_data_X[:,32:] to train the model with two point corrlators, change the input shape below as input_shape=[32,]

### 3. Normalization
normalizer = layers.Normalization(input_shape=[64,], axis=None)
normalizer.adapt(loaded_data_X)

### 4. Model
model = tf.keras.Sequential([
    normalizer,
    layers.Dense(1024, activation='relu'),
    layers.Dense(2*1024, activation='relu'),
    layers.Dense(1024, activation='relu'),
    layers.Dense(units=1)])

model.compile(
    optimizer=tf.optimizers.Adam(learning_rate=0.00005),
    loss='mean_absolute_error')

history = model.fit(
    trainX,
    trainY,
    epochs=500,
    # Suppress logging.
    verbose=0,
    # Calculate validation results on 20% of the training data.#,batch_size=64
    validation_split = 0.2)

### 5. Losses
def plot_loss(history):
    plt.plot(history.history['loss'], label='loss',linewidth=3.0)
    plt.plot(history.history['val_loss'], label='val_loss',linewidth=3.0)
    plt.xlabel('Epoch',fontsize=20)
    plt.ylabel('Error',fontsize=20)
    plt.xticks(fontsize=20)
    plt.yticks(fontsize=20)
    plt.legend(fontsize=20)
    plt.ylim([0,0.05])
    plt.grid(True)
plot_loss(history)

### 6. Accuracy
y = model.predict(testX)
compare = tf.keras.metrics.MeanAbsoluteError()
compare.update_state(testY, y)
compare.result().numpy()

### 7. Saving model
model.save('my_model')

### 8. Scatter plot
plt.scatter(testY, y, marker=".",linewidth=3.0)
plt.plot(np.linspace(0,max(testY)),np.linspace(0,max(testY)),linewidth=3.0,color='red')
plt.xticks(fontsize=20);
plt.yticks(fontsize=20);
plt.xlabel(r'$S_{{\rm{corr}}}^{{\rm{test}}}/N$',fontsize=20);
plt.ylabel(r'$S_{{\rm{corr}}}^{{\rm{pred}}}/N$',fontsize=20);
plt.savefig('corr_entropy.pdf',bbox_inches='tight')
