### 1. Libraries
import matplotlib.pyplot as plt
import numpy as np
from scipy import stats

import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers

from sklearn.model_selection import train_test_split
from sklearn.utils import shuffle

### 2. Loading Data
loaded_data_X0= np.load("my_data_X.npy")
loaded_data_Y = np.load("my_data_Y.npy")

### 3. Model
rates=np.linspace(0.00,0.05,NN) # list of rates of random noise (omega in paper)
data=[] # list to be used for storing density as a function of omega
NN=100
for zz in range(0,NN):
    
    ################Add noise to the data#################
    rand_matrix=np.random.random(loaded_data_X0.shape)-np.random.random(loaded_data_X0.shape) 

    noise_rate=rates[zz]
    loaded_data_X=loaded_data_X0+noise_rate*rand_matrix
    
    trainX, testX, trainY, testY=train_test_split(loaded_data_X, loaded_data_Y, test_size=0.2, random_state=42)
    
    ################Construct neural network layers#################
    normalizer = layers.Normalization(input_shape=[64,], axis=None)
    normalizer.adapt(trainX)
    
    model = tf.keras.Sequential([
    normalizer,
    layers.Dense(1024, activation='relu'),
    layers.Dense(2*1024, activation='relu'),
    layers.Dense(1024, activation='relu'),
    layers.Dense(units=1)])
    
    model.compile(
    optimizer=tf.optimizers.Adam(learning_rate=0.00005),
    loss='mean_absolute_error')
    
    ################Train the Model#################
    history = model.fit(
    trainX,
    trainY,
    epochs=500,
    # Suppress logging.
    verbose=0,
    # Calculate validation results on 20% of the training data.#,batch_size=64
    validation_split = 0.2)
    
    ################Predicting Test Results#################
    y = model.predict(testX)
    
    ################Generate Density#################
    kernel = stats.gaussian_kde(np.ravel(testY-y),bw_method=0.4)
    data.append(np.array([noise_rate*np.ones(500),np.linspace(-0.05,0.05,500),kernel(np.linspace(-0.05,0.05,500))]))

### 4. Saving data 
for kk in range(0,NN):
    if kk==0:
        data_array=data[0]
    else:
        data_array=np.concatenate((data_array,data[kk]),axis=1)

np.save("data.npy", data_array)
