"""
Created for generating UV-phase diagram

@author: Faluke Aikebaier
"""

### 1. Libraries
import numpy as np
import matplotlib.pyplot as plt
import math as m
import tensorflow as tf
from tensorflow import keras
import seaborn as sns
import pandas as pd
from scipy.interpolate import griddata

### 2. Loading data and model
dataUV=np.load("dataUV_UV.npy")
dataX=np.load("dataX_UV.npy")
dataY=np.load("dataY_UV.npy")

my_model = tf.keras.models.load_model('my_model')
predictY=my_model.predict(dataX)

### 3. Reorganizing data for imshow
data_true=np.concatenate([dataUV,dataY],axis=1)
data_pred=np.concatenate([dataUV,predictY],axis=1)

df_true=pd.DataFrame(data_true,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
ndata_true=df_true.to_numpy()

df_pred=pd.DataFrame(data_pred,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
ndata_pred=df_pred.to_numpy()

### 4. Data gridding
# define grid.
xi = np.linspace(-8,8,1000)
yi = np.linspace(-6,6,1000)

# grid the data.ndata
zi_true = griddata(points=(ndata_true[:,0], ndata_true[:,1]), values=ndata_true[:,2], xi=(xi[None,:], yi[:,None]), method='nearest')
zi_pred = griddata(points=(ndata_pred[:,0], ndata_pred[:,1]), values=ndata_pred[:,2], xi=(xi[None,:], yi[:,None]), method='nearest')

### 5. Plotting
f = plt.figure(figsize=(10,14))
f.add_subplot(121)
plt.imshow(zi_true, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi.min(), yi.max()],origin='lower')
plt.xlabel(r'$U/t$',fontsize=22);
plt.ylabel(r'$V/t$',fontsize=22);
plt.xticks([-8, -4, 0, 4, 8],fontsize=22);
plt.yticks([-6, -3, 0, 3, 6],fontsize=22);
f.add_subplot(122)
plt.imshow(zi_pred, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi.min(), yi.max()],origin='lower')
plt.xlabel(r'$U/t$',fontsize=22);
plt.ylabel(r'$V/t$',fontsize=22);
plt.xticks([-8, -4, 0, 4, 8],fontsize=22);
plt.yticks([-6, -3, 0, 3, 6],fontsize=22);

plt.savefig('UVphase.pdf',bbox_inches='tight');


