"""
Created for generating phase diagrams and various parameter dependence of the correlation entropy

@author: Faluke Aikebaier
"""

### 1. Libraries
import numpy as np
import matplotlib.pyplot as plt
import math as m
import tensorflow as tf
from tensorflow import keras
import seaborn as sns
import pandas as pd
from scipy.interpolate import griddata

### 2. Loading model and data
# loading model
my_model = tf.keras.models.load_model('my_model')

# data for phase diagrams
dataUV_UV=np.load("dataUV_UV.npy")
dataX_UV=np.load("dataX_UV.npy")
dataY_UV=np.load("dataY_UV.npy")

dataUV_Ulambda=np.load("dataUV_Ulambda.npy")
dataX_Ulambda=np.load("dataX_Ulambda.npy")
dataY_Ulambda=np.load("dataY_Ulambda.npy")

dataUV_Uh=np.load("dataUV_Uh.npy")
dataX_Uh=np.load("dataX_Uh.npy")
dataY_Uh=np.load("dataY_Uh.npy")

dataUV_Umu=np.load("dataUV_Umu.npy")
dataX_Umu=np.load("dataX_Umu.npy")
dataY_Umu=np.load("dataY_Umu.npy")

predictY_UV=my_model.predict(dataX_UV)
predictY_Ulambda=my_model.predict(dataX_Ulambda)
predictY_Uh=my_model.predict(dataX_Uh)
predictY_Umu=my_model.predict(dataX_Umu)

# data for parameter dependence
## Dependence of U for various lambda (strength of SOC)
data_U_Ulambda0 = np.load("dataU_Ulambda0.npy")
data_X_Ulambda0 = np.load("dataX_Ulambda0.npy")
data_Y_Ulambda0 = np.load("dataY_Ulambda0.npy")

data_U_Ulambda04 = np.load("dataU_Ulambda04.npy")
data_X_Ulambda04 = np.load("dataX_Ulambda04.npy")
data_Y_Ulambda04 = np.load("dataY_Ulambda04.npy")

data_U_Ulambda12 = np.load("dataU_Ulambda12.npy")
data_X_Ulambda12 = np.load("dataX_Ulambda12.npy")
data_Y_Ulambda12 = np.load("dataY_Ulambda12.npy")

## Dependence of lambda for various U
data_U_lambdaU2 = np.load("dataU_lambdaU2.npy")
data_X_lambdaU2 = np.load("dataX_lambdaU2.npy")
data_Y_lambdaU2 = np.load("dataY_lambdaU2.npy")

data_U_lambdaU4 = np.load("dataU_lambdaU4.npy")
data_X_lambdaU4 = np.load("dataX_lambdaU4.npy")
data_Y_lambdaU4 = np.load("dataY_lambdaU4.npy")

data_U_lambdaU8 = np.load("dataU_lambdaU8.npy")
data_X_lambdaU8 = np.load("dataX_lambdaU8.npy")
data_Y_lambdaU8 = np.load("dataY_lambdaU8.npy")

## Dependence of U for various h (magnetic field)
data_Uh0 = np.load("dataU_Uh0.npy")
data_Xh0 = np.load("dataX_Uh0.npy")
data_Yh0 = np.load("dataY_Uh0.npy")

data_Uh4 = np.load("dataU_Uh04.npy")
data_Xh4 = np.load("dataX_Uh04.npy")
data_Yh4 = np.load("dataY_Uh04.npy")

data_Uh8 = np.load("dataU_Uh08.npy")
data_Xh8 = np.load("dataX_Uh08.npy")
data_Yh8 = np.load("dataY_Uh08.npy")

## Dependence of h for various U
data_hU2 = np.load("dataU_hU2.npy")
data_hX2 = np.load("dataX_hU2.npy")
data_hY2 = np.load("dataY_hU2.npy")

data_hU4 = np.load("dataU_hU4.npy")
data_hX4 = np.load("dataX_hU4.npy")
data_hY4 = np.load("dataY_hU4.npy")

data_hU8 = np.load("dataU_hU8.npy")
data_hX8 = np.load("dataX_hU8.npy")
data_hY8 = np.load("dataY_hU8.npy")

## Dependence of U for various mu (chemical potential)
data_Umu0 = np.load("dataU_Umu0.npy")
data_Xmu0 = np.load("dataX_Umu0.npy")
data_Ymu0 = np.load("dataY_Umu0.npy")

data_Umu08 = np.load("dataU_Umu08.npy")
data_Xmu08 = np.load("dataX_Umu08.npy")
data_Ymu08 = np.load("dataY_Umu08.npy")

data_Umu16 = np.load("dataU_Umu16.npy")
data_Xmu16 = np.load("dataX_Umu16.npy")
data_Ymu16 = np.load("dataY_Umu16.npy")

## Dependence of mu for various U
data_muU2 = np.load("dataU_muU2.npy")
data_muX2 = np.load("dataX_muU2.npy")
data_muY2 = np.load("dataY_muU2.npy")

data_muU4 = np.load("dataU_muU4.npy")
data_muX4 = np.load("dataX_muU4.npy")
data_muY4 = np.load("dataY_muU4.npy")

data_muU6 = np.load("dataU_muU6.npy")
data_muX6 = np.load("dataX_muU6.npy")
data_muY6 = np.load("dataY_muU6.npy")

predict_YUlambda0=my_model.predict(data_X_Ulambda0)
predict_YUlambda04=my_model.predict(data_X_Ulambda04 )
predict_YUlambda12=my_model.predict(data_X_Ulambda12)

predict_YlambdaU2=my_model.predict(data_X_lambdaU2)
predict_YlambdaU4=my_model.predict(data_X_lambdaU4)
predict_YlambdaU8=my_model.predict(data_X_lambdaU8)

predict_Yh0=my_model.predict(data_Xh0)
predict_Yh4=my_model.predict(data_Xh4)
predict_Yh8=my_model.predict(data_Xh8)

predict_hY2=my_model.predict(data_hX2)
predict_hY4=my_model.predict(data_hX4)
predict_hY8=my_model.predict(data_hX8)

predict_Ymu0=my_model.predict(data_Xmu0)
predict_Ymu08=my_model.predict(data_Xmu08)
predict_Ymu16=my_model.predict(data_Xmu16)

predict_muY2=my_model.predict(data_muX2)
predict_muY4=my_model.predict(data_muX4)
predict_muY6=my_model.predict(data_muX6)

### 3. Reorganizing phase diagrams data for imshow
UV_true=np.concatenate([dataUV_UV,dataY_UV],axis=1)
UV_pred=np.concatenate([dataUV_UV,predictY_UV],axis=1)

Ulambda_true=np.concatenate([dataUV_Ulambda,dataY_Ulambda],axis=1)
Ulambda_pred=np.concatenate([dataUV_Ulambda,predictY_Ulambda],axis=1)

Uh_true=np.concatenate([dataUV_Uh,dataY_Uh],axis=1)
Uh_pred=np.concatenate([dataUV_Uh,predictY_Uh],axis=1)

Umu_true=np.concatenate([dataUV_Umu,dataY_Umu],axis=1)
Umu_pred=np.concatenate([dataUV_Umu,predictY_Umu],axis=1)

df_true_UV=pd.DataFrame(UV_true,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])
df_pred_UV=pd.DataFrame(UV_pred,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])

df_true_Ulambda=pd.DataFrame(Ulambda_true,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])
df_pred_Ulambda=pd.DataFrame(Ulambda_pred,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])

df_true_Uh=pd.DataFrame(Uh_true,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])
df_pred_Uh=pd.DataFrame(Uh_pred,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])

df_true_Umu=pd.DataFrame(Umu_true,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])
df_pred_Umu=pd.DataFrame(Umu_pred,columns = [r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$'])

df_true_UV_2=df_true_UV.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
df_pred_UV_2=df_pred_UV.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)

df_true_Ulambda_2=df_true_Ulambda.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
df_pred_Ulambda_2=df_pred_Ulambda.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)

df_true_Uh_2=df_true_Uh.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
df_pred_Uh_2=df_pred_Uh.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)

df_true_Umu_2=df_true_Umu.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)
df_pred_Umu_2=df_pred_Umu.drop_duplicates(subset=[r'$U/t$',r'$V/t$'],keep=False).sort_values(by=[r'$U/t$',r'$V/t$',r'$\frac{S_{\rm{corr}}^{\rm{true}}}{N}$']).reset_index(drop=True)

UVdata_true=df_true_UV_2.to_numpy()
UVdata_pred=df_pred_UV_2.to_numpy()

Ulambdadata_true=df_true_Ulambda_2.to_numpy()
Ulambdadata_pred=df_pred_Ulambda_2.to_numpy()

Uhdata_true=df_true_Uh_2.to_numpy()
Uhdata_pred=df_pred_Uh_2.to_numpy()

Umudata_true=df_true_Umu_2.to_numpy()
Umudata_pred=df_pred_Umu_2.to_numpy()

### 4. Data gridding
# define grid.
xi = np.linspace(-8,8,1000)
yi_UV = np.linspace(-6,6,1000)
yi_Ulambda = np.linspace(0,1.5,1000)
yi_Uh = np.linspace(0,2,1000)
yi_Umu = np.linspace(0,2,1000)

# grid the data.ndata
zi_UV_true = griddata(points=(UVdata_true[:,0], UVdata_true[:,1]), values=UVdata_true[:,2], xi=(xi[None,:], yi_UV[:,None]), method='nearest')
zi_UV_pred = griddata(points=(UVdata_pred[:,0], UVdata_pred[:,1]), values=UVdata_pred[:,2], xi=(xi[None,:], yi_UV[:,None]), method='nearest')

zi_Ulambda_true = griddata(points=(Ulambdadata_true[:,0], Ulambdadata_true[:,1]), values=Ulambdadata_true[:,2], xi=(xi[None,:], yi_Ulambda[:,None]), method='nearest')
zi_Ulambda_pred = griddata(points=(Ulambdadata_pred[:,0], Ulambdadata_pred[:,1]), values=Ulambdadata_pred[:,2], xi=(xi[None,:], yi_Ulambda[:,None]), method='nearest')

zi_Uh_true = griddata(points=(Uhdata_true[:,0], Uhdata_true[:,1]), values=Uhdata_true[:,2], xi=(xi[None,:], yi_Uh[:,None]), method='nearest')
zi_Uh_pred = griddata(points=(Uhdata_pred[:,0], Uhdata_pred[:,1]), values=Uhdata_pred[:,2], xi=(xi[None,:], yi_Uh[:,None]), method='nearest')

zi_Umu_true = griddata(points=(Umudata_true[:,0], Umudata_true[:,1]), values=Umudata_true[:,2], xi=(xi[None,:], yi_Umu[:,None]), method='nearest')
zi_Umu_pred = griddata(points=(Umudata_pred[:,0], Umudata_pred[:,1]), values=Umudata_pred[:,2], xi=(xi[None,:], yi_Umu[:,None]), method='nearest')

### 5. Plotting
font_size=18

f = plt.figure(figsize=(16,9))
f.add_subplot(341)
plt.plot(data_U_Ulambda0,data_Y_Ulambda0,linewidth=3,label=r'$\lambda_R=0$', color='tab:blue')
plt.plot(data_U_Ulambda0,predict_YUlambda0,'--',linewidth=3, color='k')
plt.plot(data_U_Ulambda0,data_Y_Ulambda04,linewidth=3,label=r'$\lambda_R=0.4t$', color='tab:orange')
plt.plot(data_U_Ulambda0,predict_YUlambda04,'--',linewidth=3, color='k')
plt.plot(data_U_Ulambda0,data_Y_Ulambda12,linewidth=3,label=r'$\lambda_R=1.2t$', color='tab:green')
plt.plot(data_U_Ulambda0,predict_YUlambda12,'--',linewidth=3, color='k')
plt.xlim([0,10]);
plt.legend(fontsize=12,loc='lower right');
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks(fontsize=font_size);
plt.yticks([0,0.2,0.4,0.6],fontsize=font_size);

f.add_subplot(342)
plt.plot(data_U_lambdaU2,data_Y_lambdaU2,linewidth=3,label=r'$U=2t$', color='tab:blue')
plt.plot(data_U_lambdaU2,predict_YlambdaU2,'--',linewidth=3, color='k')
plt.plot(data_U_lambdaU2,data_Y_lambdaU4,linewidth=3,label=r'$U=4t$', color='tab:orange')
plt.plot(data_U_lambdaU2,predict_YlambdaU4,'--',linewidth=3, color='k')
plt.plot(data_U_lambdaU2,data_Y_lambdaU8,linewidth=3,label=r'$U=8t$', color='tab:green')
plt.plot(data_U_lambdaU2,predict_YlambdaU8,'--',linewidth=3, color='k')
plt.legend(fontsize=10,bbox_to_anchor=(0.37, 0.4));
plt.xlabel(r'$\lambda_R/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks([0,0.4,0.8,1.2],fontsize=font_size);
plt.yticks(fontsize=font_size);

f.add_subplot(343)
ax=plt.imshow(zi_Ulambda_true, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Ulambda.min(), yi_Ulambda.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$\lambda_R/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1.0,1.5],fontsize=font_size);

f.add_subplot(344)
ax=plt.imshow(zi_Ulambda_pred, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Ulambda.min(), yi_Ulambda.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$\lambda_R/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1.0,1.5],fontsize=font_size);

f.add_subplot(345)
plt.plot(data_Uh0,data_Yh0,linewidth=3,label=r'$h_z=0$', color='tab:blue')
plt.plot(data_Uh0,predict_Yh0,'--',linewidth=3, color='k')
plt.plot(data_Uh0,data_Yh4,linewidth=3,label=r'$h_z=0.4t$', color='tab:orange')
plt.plot(data_Uh0,predict_Yh4,'--',linewidth=3, color='k')
plt.plot(data_Uh0,data_Yh8,linewidth=3,label=r'$h_z=0.8t$', color='tab:green')
plt.plot(data_Uh0,predict_Yh8,'--',linewidth=3, color='k')
plt.xlim([0,10]);
plt.legend(fontsize=12,bbox_to_anchor=(0.4, 0.32));
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks([0,2,4,6,8,10],fontsize=font_size);
plt.yticks([0,0.2,0.4,0.6],fontsize=font_size);

f.add_subplot(346)
plt.plot(data_hU2,data_hY2,linewidth=3,label=r'$U=2t$', color='tab:blue')
plt.plot(data_hU2,predict_hY2,'--',linewidth=3, color='k')
plt.plot(data_hU2,data_hY4,linewidth=3,label=r'$U=4t$', color='tab:orange')
plt.plot(data_hU2,predict_hY4,'--',linewidth=3, color='k')
plt.plot(data_hU2,data_hY8,linewidth=3,label=r'$U=8t$', color='tab:green')
plt.plot(data_hU2,predict_hY8,'--',linewidth=3, color='k')
plt.xlim([0,2]);
plt.legend(fontsize=12);
plt.xlabel(r'$h_z/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks([0,0.5,1,1.5,2],fontsize=font_size);
plt.yticks([0,0.2,0.4,0.6],fontsize=font_size);

f.add_subplot(347)
ax=plt.imshow(zi_Uh_true, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Uh.min(), yi_Uh.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$h_z/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1,1.5,2],fontsize=font_size);

f.add_subplot(348)
ax=plt.imshow(zi_Uh_pred, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Uh.min(), yi_Uh.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$h_z/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1,1.5,2],fontsize=font_size);

f.add_subplot(349)
plt.plot(data_Umu0,data_Ymu0,linewidth=3,label=r'$\mu=0$', color='tab:blue')
plt.plot(data_Umu0,predict_Ymu0,'--',linewidth=3, color='k')
plt.plot(data_Umu0,data_Ymu08,linewidth=3,label=r'$\mu=0.4t$', color='tab:orange')
plt.plot(data_Umu0,predict_Ymu08,'--',linewidth=3, color='k')
plt.plot(data_Umu0,data_Ymu16,linewidth=3,label=r'$\mu=0.8t$', color='tab:green')
plt.plot(data_Umu0,predict_Ymu16,'--',linewidth=3, color='k')
plt.xlim([-10,10]);
#plt.legend(fontsize=9.9,bbox_to_anchor=(0.5, 0.575));
plt.legend(fontsize=10,loc='upper center');
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks([-10,-5,0,5,10],fontsize=font_size);
plt.yticks([0,0.2,0.4,0.6],fontsize=font_size);

f.add_subplot(3,4,10)
plt.plot(data_muU2,data_muY2,linewidth=3,label=r'$U=2t$', color='tab:blue')
plt.plot(data_muU2,predict_muY2,'--',linewidth=3, color='k')
plt.plot(data_muU2,data_muY4,linewidth=3,label=r'$U=4t$', color='tab:orange')
plt.plot(data_muU2,predict_muY4,'--',linewidth=3, color='k')
plt.plot(data_muU2,data_muY6,linewidth=3,label=r'$U=6t$', color='tab:green')
plt.plot(data_muU2,predict_muY6,'--',linewidth=3, color='k')
plt.xlim([0,2]);
plt.legend(fontsize=10,bbox_to_anchor=(0.36, 0.5));
#plt.legend(fontsize=12,loc='upper center');
plt.xlabel(r'$\mu/t$',fontsize=font_size);
plt.ylabel(r'$S_{\rm{corr}}/N$',fontsize=font_size);
plt.xticks([0,0.5,1,1.5,2],fontsize=font_size);
plt.yticks(fontsize=font_size);

f.add_subplot(3,4,11)
ax=plt.imshow(zi_Umu_true, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Umu.min(), yi_Umu.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$\mu/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1,1.5,2],fontsize=font_size);

f.add_subplot(3,4,12)
ax=plt.imshow(zi_Umu_pred, cmap='viridis',vmin = 0, vmax = np.log(2), \
           extent =[xi.min(), xi.max(), yi_Umu.min(), yi_Umu.max()],origin='lower', aspect='auto')
plt.xlabel(r'$U/t$',fontsize=font_size);
plt.ylabel(r'$\mu/t$',fontsize=font_size);
plt.xticks([-8,-4,0,4,8],fontsize=font_size);
plt.yticks([0,0,0.5,1,1.5,2],fontsize=font_size);

f.tight_layout()
#plt.subplots_adjust(left=0.1, bottom=0.1, right=0.9, top=0.9, wspace=0.4, hspace=0.5)

plt.savefig('prediction.pdf',bbox_inches='tight');


